import * as angular from "angular";
import "kendo.datepicker";
import * as moment from "moment";

import { keyPressModule } from "./keyPress/keyPress.directive";

const template = require<string>("./intervalPicker.html");
import "./intervalPicker.scss";

export const intervalPickerModule = "seges.regnet.components.intervalPicker";

import {
    IntervalPickerBindings,
    PublicDates,
} from "./intervalPicker.bindings";

export type Preset = "day" | "week" | "month" | "year" | "custom";

export class IntervalPicker implements ng.IComponentOptions, IntervalPickerBindings {
    defaultFormat: string = "DD-MM-YYYY";
    format: string;
    canBeEmpty: boolean;
    startDate: Date;
    endDate: Date;
    publicDates: PublicDates;
    publicPreset: Preset;
    onOkCallback: Function;
    onCancelCallback: Function;

    startOptions: kendo.ui.DatePickerOptions;
    endOptions: kendo.ui.DatePickerOptions;
    startPicker: kendo.ui.DatePicker;
    endPicker: kendo.ui.DatePicker;

    preset: Preset;
    showSelector: boolean;

    constructor(private $rootScope: ng.IRootScopeService) { }

    $onInit(): void {
        this.startOptions = {
            change: () => this.onStartDateChanged(),
            disableDates: (date: Date) => this.isDateDisabled(date),
        };

        this.endOptions = {
            change: () => this.onEndDateChanged(),
            disableDates: (date: Date) => {
                return moment(date).isBefore(this.startDate);
            },
        };

        if (!this.publicDates.start) {
            this.initInternalDates();
        }
        if (!this.publicPreset) {
            this.setPreset("day");
        } else {
            this.preset = this.publicPreset;
        }

        this.$rootScope.$watchGroup([
            () => { return this.publicDates.start; },
            () => { return this.publicDates.end; },
        ], (oldVal: Date, newVal: Date) => {
            this.initInternalDates();
        });
    }

    /**
     * Returns true if date is currently disabled.
     * @param date {Date} The date to test.
     */
    isDateDisabled(date: Date): boolean {
        const momentInstance = moment(date);

        if (this.isPreset("month")) {
            return momentInstance.isAfter(this.endDate) && momentInstance.date() !== 1;
        } else if (this.isPreset("week")) {
            return momentInstance.isAfter(this.endDate) && momentInstance.day() !== 0;
        } else if (this.isPreset("day")) {
            return false;
        }

        return momentInstance.isAfter(this.endDate);
    }

    /**
     * Set start and end dates given a year offset.
     * @param yearOffset {number} The number of years to offset. Positive = future, negative = past.
     */
    setYear(yearOffset: number): void {
        if (yearOffset === 0) {
            this.startDate = moment().toDate();
        }

        const currentYear = moment(this.startDate).year() + yearOffset;
        const newStartDate = moment(this.startDate).year(currentYear);
        const newEndDate = moment(newStartDate).year(newStartDate.year() + 1);

        this.setDates(newStartDate, newEndDate);

    }

    /**
     * Set start and end dates given a month offset.
     * @param monthOffset {number} The number of month to offset. Positive = future, negative = past.
     */
    setMonth(monthOffset: number): void {
        if (monthOffset === 0) {
            this.startDate = moment().toDate();
        }

        const currentMonth = moment(this.startDate).month() + monthOffset;
        const newStartDate = moment(this.startDate).month(currentMonth).startOf("month");
        const newEndDate = moment(newStartDate).endOf("month");

        this.setDates(newStartDate, newEndDate);
    }

    /**
     * Set start and end dates given a week offset.
     * @param weekOffset {number} The number of weeks to offset. Positive = future, negative = past.
     */
    setWeek(weekOffset: number): void {
        if (weekOffset === 0) {
            this.startDate = moment().toDate();
        }

        const currentWeek = moment(this.startDate).isoWeek() + weekOffset;
        const newStartDate = moment().year(this.startDate.getFullYear()).isoWeek(currentWeek).startOf("isoWeek");
        const newEndDate = moment(newStartDate).endOf("isoWeek");

        this.setDates(newStartDate, newEndDate);
    }

    /**
     * Set start date given a day offset. Clears end date.
     * @param dayOffset {number} The number of days to offset. Positive = future, negative = past.
     */
    setDay(dayOffset: number): void {
        if (dayOffset === 0) {
            this.startDate = moment().toDate();
        }

        const newStartDate = moment(this.startDate).add(dayOffset, "day");
        this.setDates(newStartDate);
    }

    /**
     * Set start and end dates given an interval length from current date.
     * @param intervalLength {number} The interval length to offset. Positive = future, negative = past.
     */
    setInterval(intervalLength: number): void {
        const newStartDate = moment().subtract(intervalLength, "days");
        const newEndDate = moment();

        this.setDates(newStartDate, newEndDate);
    }

    /**
     * Set shown preset tab.
     * @param preset {Preset} The preset to show.
     */
    setPreset(preset: Preset): void {
        switch (preset) {
            case "day":
                this.preset = preset;
                this.endDate = undefined;
                break;
            case "week":
                this.preset = preset;
                this.setWeek(0);
                break;
            case "month":
                this.preset = preset;
                this.setMonth(0);
                break;
            case "year":
                this.preset = preset;
                this.setYear(0);
                break;
            case "custom":
                this.preset = preset;
                break;
            default:
                throw Error("Preset value not valid");
        }
        this.publicPreset = this.preset;
        this.drawPickers();
    }

    /**
     * Return true if preset is currently shown.
     * @param preset {Preset} The preset to test against.
     */
    isPreset(preset: Preset): boolean {
        return this.preset === preset;
    }

    /**
     * Return true if dates are valid for preset.
     */
    isValid(): boolean {
        if (this.isPreset("day") && this.startDate) {
            return true;
        }

        return !!this.startDate && !!this.endDate;
    }

    formatDate(date: Date): string {
        const format = this.format || this.defaultFormat;
        return moment(date).format(format);
    }

    toggleSelector(): void {
        this.showSelector = !this.showSelector;
        setTimeout(() => angular.element(".interval-picker").focus(), 100);
    }

    initInternalDates(): void {
        if (this.publicDates.start) {
            this.setDates(moment(this.publicDates.start));
        } else if (this.canBeEmpty) {
            return;
        } else {
            this.setDates(moment());
        }

        if (this.publicDates.end) {
            this.setDates(undefined, moment(this.publicDates.end));
        }

        if (!this.publicDates.start && !this.publicDates.end) {
            this.onOk();
        }
    }

    updatePublicDates(): void {
        if (this.startDate) {
            this.publicDates.start = this.startDate;

        }
        if (this.endDate) {
            this.publicDates.end = this.endDate;
        } else {
            this.publicDates.end = undefined;
        }
    }

    /**
     * Handle OK button click.
     */
    onOk(): void {
        if (this.isValid() === true) {
            this.showSelector = false;
            this.updatePublicDates();

            if (this.onOkCallback && typeof this.onOkCallback === "function") {
                this.onOkCallback();
            }
        }
    }

    /**
     * Handle Cancel button click.
     */
    onCancel(): void {
        this.showSelector = false;

        if (this.onCancelCallback && typeof this.onCancelCallback === "function") {
            this.onCancelCallback();
        }
    }

    /**
     * Handle start date selection change.
     */
    private onStartDateChanged(): void {
        this.handleStartDateAutoFill();
        this.$rootScope.$apply();
    }

    /**
     * Handle end date selection change.
     */
    private onEndDateChanged(): void {
        this.handleEndDateAutoFill();
        this.$rootScope.$apply();
    }

    /**
     * Autofill start and end dates according to preset and selected start date.
     */
    private handleStartDateAutoFill(): void {
        if (this.isPreset("month")) {
            const dayOfMonth = moment(this.startDate).date();
            if (dayOfMonth === 1) {
                const newEndDate = moment(this.startDate).endOf("month");
                this.setDates(undefined, newEndDate);
            } else {
                this.setPreset("custom");
            }
        } else if (this.isPreset("week")) {
            const dayOfWeek = moment(this.startDate).day();
            if (dayOfWeek === 0) {
                const newEndDate = moment(this.startDate).endOf("week");
                this.setDates(undefined, newEndDate);
            } else {
                this.setPreset("custom");
            }
        }

        this.drawPickers();
    }

    /**
     * Autofill start and end dates according to preset and selected end date.
     */
    private handleEndDateAutoFill(): void {
        if (this.isPreset("month")) {
            const endMoment = moment(this.endDate);
            const dayOfMonth = endMoment.date();
            const daysInMonth = endMoment.daysInMonth();
            if (dayOfMonth !== daysInMonth) {
                this.setPreset("custom");
            }
        } else if (this.isPreset("week")) {
            const endMoment = moment(this.endDate);
            const dayOfWeek = endMoment.day();
            if (dayOfWeek !== 6) {
                this.setPreset("custom");
            }
        }

        this.drawPickers();
    }

    /**
     * Set dates.
     * @param startDate {Moment} The start date.
     * @param endDate {Moment} The end date.
     */
    private setDates(startDate?: moment.Moment, endDate?: moment.Moment): void {
        if (startDate) {
            this.startDate = startDate.toDate();
        }
        if (endDate) {
            this.endDate = endDate.toDate();
        }

        this.drawPickers();
    }

    /**
     * Draw kendo date pickers.
     */
    private drawPickers(): void {
        if (this.startPicker) {
            this.startPicker.setOptions(this.startOptions);
        }

        if (this.endPicker) {
            this.endPicker.setOptions(this.endOptions);
        }
    }
}

angular
    .module(intervalPickerModule, [keyPressModule])
    .component("intervalPicker", {
        controller: IntervalPicker,
        controllerAs: "intPicker",
        template,
        bindings: {
            canBeEmpty: "<",
            format: "<",
            publicDates: "=dates",
            publicPreset: "=preset",
            onOkCallback: "&onOk",
            onCancelCallback: "&onCancel",
        },
    });
